@@ -6,8 +6,14 @@ import sys
 import textwrap
 import types
 from ast import FunctionDef, Module, stmt
+from dataclasses import dataclass
+from functools import lru_cache
 from typing import Any, AnyStr, Callable, ForwardRef, NewType, TypeVar, get_type_hints
 
+from docutils.frontend import OptionParser
+from docutils.nodes import Node
+from docutils.parsers.rst import Parser as RstParser
+from docutils.utils import new_document
 from sphinx.application import Sphinx
 from sphinx.config import Config
 from sphinx.environment import BuildEnvironment
@@ -17,6 +23,7 @@ from sphinx.util import logging
 from sphinx.util.inspect import signature as sphinx_signature
 from sphinx.util.inspect import stringify_signature
 
+from .attributes_patch import patch_attribute_handling
 from .version import __version__
 
 _LOGGER = logging.getLogger(__name__)
@@ -588,74 +595,181 @@ def _inject_types_to_docstring(
     name: str,
     lines: list[str],
 ) -> None:
-
     if signature is not None:
-        for arg_name in signature.parameters:
-            annotation = type_hints.get(arg_name, None)
+        _inject_signature(type_hints, signature, app, lines)
+    if "return" in type_hints:
+        _inject_rtype(type_hints, original_obj, app, what, name, lines)
 
-            default = signature.parameters[arg_name].default
 
-            if arg_name.endswith("_"):
-                arg_name = f"{arg_name[:-1]}\\_"
+def _inject_signature(
+    type_hints: dict[str, Any],
+    signature: inspect.Signature,
+    app: Sphinx,
+    lines: list[str],
+) -> None:
+    for arg_name in signature.parameters:
+        annotation = type_hints.get(arg_name, None)
 
-            insert_index = None
-            for at, line in enumerate(lines):
-                if _line_is_param_line_for_arg(line, arg_name):
-                    # Get the arg_name from the doc to match up for type in case it has a star prefix.
-                    # Line is in the correct format so this is guaranteed to return tuple[str, str].
-                    _, arg_name = _get_sphinx_line_keyword_and_argument(line)  # type: ignore[assignment, misc]
-                    insert_index = at
-                    break
+        default = signature.parameters[arg_name].default
 
-            if annotation is not None and insert_index is None and app.config.always_document_param_types:
-                lines.append(f":param {arg_name}:")
-                insert_index = len(lines)
+        if arg_name.endswith("_"):
+            arg_name = f"{arg_name[:-1]}\\_"
 
-            if insert_index is not None:
-                if annotation is None:
-                    type_annotation = f":type {arg_name}: "
-                else:
-                    formatted_annotation = format_annotation(annotation, app.config)
-                    type_annotation = f":type {arg_name}: {formatted_annotation}"
-
-                if app.config.typehints_defaults:
-                    formatted_default = format_default(app, default, annotation is not None)
-                    if formatted_default:
-                        if app.config.typehints_defaults.endswith("after"):
-                            lines[insert_index] += formatted_default
-                        else:  # add to last param doc line
-                            type_annotation += formatted_default
-
-                lines.insert(insert_index, type_annotation)
-
-    if "return" in type_hints and not inspect.isclass(original_obj) and not inspect.isdatadescriptor(original_obj):
-        if what == "method" and name.endswith(".__init__"):  # avoid adding a return type for data class __init__
-            return
-        formatted_annotation = format_annotation(type_hints["return"], app.config)
-        insert_index = len(lines)
+        insert_index = None
         for at, line in enumerate(lines):
-            if line.startswith(":rtype:"):
-                insert_index = None
-                break
-            elif line.startswith(":return:") or line.startswith(":returns:"):
-                if " -- " in line and not app.config.typehints_use_rtype:
-                    insert_index = None
-                    break
-                insert_index = at
-            elif line.startswith(".."):
-                # Make sure that rtype comes before any usage or examples section
+            if _line_is_param_line_for_arg(line, arg_name):
+                # Get the arg_name from the doc to match up for type in case it has a star prefix.
+                # Line is in the correct format so this is guaranteed to return tuple[str, str].
+                _, arg_name = _get_sphinx_line_keyword_and_argument(line)  # type: ignore[assignment, misc]
                 insert_index = at
                 break
 
-        if insert_index is not None and app.config.typehints_document_rtype:
-            if insert_index == len(lines):  # ensure that :rtype: doesn't get joined with a paragraph of text
-                lines.append("")
-                insert_index += 1
-            if app.config.typehints_use_rtype or insert_index == len(lines):
-                lines.insert(insert_index, f":rtype: {formatted_annotation}")
+        if annotation is not None and insert_index is None and app.config.always_document_param_types:
+            lines.append(f":param {arg_name}:")
+            insert_index = len(lines)
+
+        if insert_index is not None:
+            if annotation is None:
+                type_annotation = f":type {arg_name}: "
             else:
-                line = lines[insert_index]
-                lines[insert_index] = f":return: {formatted_annotation} --{line[line.find(' '):]}"
+                formatted_annotation = format_annotation(annotation, app.config)
+                type_annotation = f":type {arg_name}: {formatted_annotation}"
+
+            if app.config.typehints_defaults:
+                formatted_default = format_default(app, default, annotation is not None)
+                if formatted_default:
+                    if app.config.typehints_defaults.endswith("after"):
+                        lines[insert_index] += formatted_default
+                    else:  # add to last param doc line
+                        type_annotation += formatted_default
+
+            lines.insert(insert_index, type_annotation)
+
+
+@dataclass
+class InsertIndexInfo:
+    insert_index: int
+    found_param: bool = False
+    found_return: bool = False
+    found_directive: bool = False
+
+
+# Sphinx allows so many synonyms...
+# See sphinx.domains.python.PyObject
+PARAM_SYNONYMS = ("param ", "parameter ", "arg ", "argument ", "keyword ", "kwarg ", "kwparam ")
+
+
+def node_line_no(node: Node) -> int | None:
+    """
+    Get the 1-indexed line on which the node starts if possible. If not, return
+    None.
+
+    Descend through the first children until we locate one with a line number or
+    return None if None of them have one.
+
+    I'm not aware of any rst on which this returns None, to find out would
+    require a more detailed analysis of the docutils rst parser source code. An
+    example where the node doesn't have a line number but the first child does
+    is all `definition_list` nodes. It seems like bullet_list and option_list
+    get line numbers, but enum_list also doesn't. *shrug*.
+    """
+    while node.line is None and node.children:
+        node = node.children[0]
+    return node.line
+
+
+def tag_name(node: Node) -> str:
+    return node.tagname  # type:ignore[attr-defined,no-any-return] # noqa: SC200
+
+
+def get_insert_index(app: Sphinx, lines: list[str]) -> InsertIndexInfo | None:
+    # 1. If there is an existing :rtype: anywhere, don't insert anything.
+    if any(line.startswith(":rtype:") for line in lines):
+        return None
+
+    # 2. If there is a :returns: anywhere, either modify that line or insert
+    #    just before it.
+    for at, line in enumerate(lines):
+        if line.startswith((":return:", ":returns:")):
+            return InsertIndexInfo(insert_index=at, found_return=True)
+
+    # 3. Insert after the parameters.
+    # To find the parameters, parse as a docutils tree.
+    settings = OptionParser(components=(RstParser,)).get_default_values()
+    settings.env = app.env
+    doc = new_document("", settings=settings)
+    RstParser().parse("\n".join(lines), doc)
+
+    # Find a top level child which is a field_list that contains a field whose
+    # name starts with one of the PARAM_SYNONYMS. This is the parameter list. We
+    # hope there is at most of these.
+    for child in doc.children:
+        if tag_name(child) != "field_list":
+            continue
+
+        if not any(c.children[0].astext().startswith(PARAM_SYNONYMS) for c in child.children):
+            continue
+
+        # Found it! Try to insert before the next sibling. If there is no next
+        # sibling, insert at end.
+        # If there is a next sibling but we can't locate a line number, insert
+        # at end. (I don't know of any input where this happens.)
+        next_sibling = child.next_node(descend=False, siblings=True)
+        line_no = node_line_no(next_sibling) if next_sibling else None
+        at = line_no - 2 if line_no else len(lines)
+        return InsertIndexInfo(insert_index=at, found_param=True)
+
+    # 4. Insert before examples
+    # TODO: Maybe adjust which tags to insert ahead of
+    for child in doc.children:
+        if tag_name(child) in ["literal_block", "paragraph", "field_list"]:
+            continue
+        line_no = node_line_no(child)
+        at = line_no - 2 if line_no else len(lines)
+        return InsertIndexInfo(insert_index=at, found_directive=True)
+
+    # 5. Otherwise, insert at end
+    return InsertIndexInfo(insert_index=len(lines))
+
+
+def _inject_rtype(
+    type_hints: dict[str, Any],
+    original_obj: Any,
+    app: Sphinx,
+    what: str,
+    name: str,
+    lines: list[str],
+) -> None:
+    if inspect.isclass(original_obj) or inspect.isdatadescriptor(original_obj):
+        return
+    if what == "method" and name.endswith(".__init__"):  # avoid adding a return type for data class __init__
+        return
+    if not app.config.typehints_document_rtype:
+        return
+
+    r = get_insert_index(app, lines)
+    if r is None:
+        return
+
+    insert_index = r.insert_index
+
+    if not app.config.typehints_use_rtype and r.found_return and " -- " in lines[insert_index]:
+        return
+
+    formatted_annotation = format_annotation(type_hints["return"], app.config)
+
+    if insert_index == len(lines) and not r.found_param:
+        # ensure that :rtype: doesn't get joined with a paragraph of text
+        lines.append("")
+        insert_index += 1
+    if app.config.typehints_use_rtype or not r.found_return:
+        line = f":rtype: {formatted_annotation}"
+        lines.insert(insert_index, line)
+        if r.found_directive:
+            lines.insert(insert_index + 1, "")
+    else:
+        line = lines[insert_index]
+        lines[insert_index] = f":return: {formatted_annotation} --{line[line.find(' '):]}"
 
 
 def validate_config(app: Sphinx, env: BuildEnvironment, docnames: list[str]) -> None:  # noqa: U100
@@ -668,6 +782,29 @@ def validate_config(app: Sphinx, env: BuildEnvironment, docnames: list[str]) ->
         raise ValueError(f"typehints_formatter needs to be callable or `None`, not {formatter}")
 
 
+@lru_cache()  # A cute way to make sure the function only runs once.
+def fix_autodoc_typehints_for_overloaded_methods() -> None:
+    """
+    sphinx-autodoc-typehints responds to the "autodoc-process-signature" event
+    to remove types from the signature line of functions.
+
+    Normally, `FunctionDocumenter.format_signature` and
+    `MethodDocumenter.format_signature` call `super().format_signature` which
+    ends up going to `Documenter.format_signature`, and this last method emits
+    the `autodoc-process-signature` event. However, if there are overloads,
+    `FunctionDocumenter.format_signature` does something else and the event
+    never occurs.
+
+    Here we remove this alternative code path by brute force.
+
+    See https://github.com/tox-dev/sphinx-autodoc-typehints/issues/296
+    """
+    from sphinx.ext.autodoc import FunctionDocumenter, MethodDocumenter
+
+    del FunctionDocumenter.format_signature
+    del MethodDocumenter.format_signature
+
+
 def setup(app: Sphinx) -> dict[str, bool]:
     app.add_config_value("always_document_param_types", False, "html")
     app.add_config_value("typehints_fully_qualified", False, "env")
@@ -681,6 +818,8 @@ def setup(app: Sphinx) -> dict[str, bool]:
     app.connect("env-before-read-docs", validate_config)  # config may be changed after “config-inited” event
     app.connect("autodoc-process-signature", process_signature)
     app.connect("autodoc-process-docstring", process_docstring)
+    fix_autodoc_typehints_for_overloaded_methods()
+    patch_attribute_handling(app)
     return {"parallel_read_safe": True}
 
 
