@@ -1,7 +1,7 @@
-import typing
 from dataclasses import dataclass
 from mailbox import Mailbox
-from typing import Callable, Optional, Union
+from types import CodeType
+from typing import Callable, Optional, Union, overload
 
 
 def get_local_function():
@@ -22,10 +22,10 @@ class Class:
     :param z: baz
     """
 
-    def __init__(self, x: bool, y: int, z: typing.Optional[str] = None) -> None:  # noqa: U100
+    def __init__(self, x: bool, y: int, z: Optional[str] = None) -> None:  # noqa: U100
         pass
 
-    def a_method(self, x: bool, y: int, z: typing.Optional[str] = None) -> str:  # noqa: U100
+    def a_method(self, x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: U100
         """
         Method docstring.
 
@@ -56,7 +56,7 @@ class Class:
         """
 
     @classmethod
-    def a_classmethod(cls, x: bool, y: int, z: typing.Optional[str] = None) -> str:  # noqa: U100
+    def a_classmethod(cls, x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: U100
         """
         Classmethod docstring.
 
@@ -66,7 +66,7 @@ class Class:
         """
 
     @staticmethod
-    def a_staticmethod(x: bool, y: int, z: typing.Optional[str] = None) -> str:  # noqa: U100
+    def a_staticmethod(x: bool, y: int, z: Optional[str] = None) -> str:  # noqa: U100
         """
         Staticmethod docstring.
 
@@ -114,7 +114,7 @@ class DummyException(Exception):  # noqa: N818
         super().__init__(message)
 
 
-def function(x: bool, y: int, z_: typing.Optional[str] = None) -> str:  # noqa: U100
+def function(x: bool, y: int, z_: Optional[str] = None) -> str:  # noqa: U100
     """
     Function docstring.
 
@@ -164,7 +164,7 @@ def function_with_typehint_comment(
     Function docstring.
 
     :parameter x: foo
-    :param y: bar
+    :parameter y: bar
     """
 
 
@@ -286,3 +286,124 @@ def func_with_examples() -> int:
 
     Here are a couple of examples of how to use this function.
     """
+
+
+@overload
+def func_with_overload(a: int, b: int) -> None:  # noqa: U100
+    ...
+
+
+@overload
+def func_with_overload(a: str, b: str) -> None:  # noqa: U100
+    ...
+
+
+def func_with_overload(a: Union[int, str], b: Union[int, str]) -> None:  # noqa: U100
+    """
+    f does the thing. The arguments can either be ints or strings but they must
+    both have the same type.
+
+    Parameters
+    ----------
+    a:
+        The first thing
+    b:
+        The second thing
+    """
+
+
+class TestClassAttributeDocs:
+    """A class"""
+
+    code: Union[CodeType, None]
+    """An attribute"""
+
+
+def func_with_examples_and_returns_after() -> int:
+    """
+    f does the thing.
+
+    Examples
+    --------
+
+    Here is an example
+
+    :returns: The index of the widget
+    """
+
+
+def func_with_parameters_and_stuff_after(a: int, b: int) -> int:  # noqa: U100
+    """A func
+
+    :param a: a tells us something
+    :param b: b tells us something
+
+    More info about the function here.
+    """
+
+
+def func_with_rtype_in_weird_spot(a: int, b: int) -> int:  # noqa: U100
+    """A func
+
+    :param a: a tells us something
+    :param b: b tells us something
+
+    Examples
+    --------
+
+    Here is an example
+
+    :returns: The index of the widget
+
+    More info about the function here.
+
+    :rtype: int
+    """
+
+
+def empty_line_between_parameters(a: int, b: int) -> int:  # noqa: U100
+    """A func
+
+    :param a: One of the following possibilities:
+
+             - a
+
+             - b
+
+             - c
+
+    :param b: Whatever else we have to say.
+
+              There is more of it And here too
+
+    More stuff here.
+    """
+
+
+def func_with_code_block() -> int:
+    """
+    A docstring.
+
+    You would say:
+
+    .. code-block::
+
+        print("some python code here")
+
+
+    .. rubric:: Examples
+
+    Here are a couple of examples of how to use this function.
+    """
+
+
+def func_with_definition_list() -> int:
+    """Some text and then a definition list.
+
+    abc
+        x
+
+    xyz
+        something
+    """
+    # See https://github.com/tox-dev/sphinx-autodoc-typehints/issues/302
