@@ -64,9 +64,9 @@ P_bound = typing_extensions.ParamSpec("P_bound", bound=str)  # type: ignore
 
 # Mypy does not support recursive type aliases, but
 # other type checkers do.
-RecList = Union[int, List["RecList"]]  # type: ignore
-MutualRecA = Union[bool, List["MutualRecB"]]  # type: ignore
-MutualRecB = Union[str, List["MutualRecA"]]  # type: ignore
+RecList = Union[int, List["RecList"]]
+MutualRecA = Union[bool, List["MutualRecB"]]
+MutualRecB = Union[str, List["MutualRecA"]]
 
 
 class A:
@@ -103,10 +103,10 @@ class Metaclass(type):
 
 class HintedMethods:
     @classmethod
-    def from_magic(cls: type[T]) -> T:
+    def from_magic(cls: type[T]) -> T:  # type: ignore
         ...
 
-    def method(self: T) -> T:
+    def method(self: T) -> T:  # type: ignore
         ...
 
 
@@ -151,7 +151,7 @@ else:
         pytest.param(Callable[..., str], "typing", "Callable", (..., str), id="Callable_returntype"),
         pytest.param(Callable[[int, str], str], "typing", "Callable", (int, str, str), id="Callable_all_types"),
         pytest.param(
-            AbcCallable[[int, str], str],  # type: ignore[misc]
+            AbcCallable[[int, str], str],  # type: ignore
             "collections.abc",
             "Callable",
             (int, str, str),
@@ -258,7 +258,7 @@ def test_parse_annotation(annotation: Any, module: str, class_name: str, args: t
             " :py:class:`~typing.TypeVar`\\(``T``)]",
         ),
         (
-            AbcCallable[[int, str], bool],  # type: ignore[misc]
+            AbcCallable[[int, str], bool],  # type: ignore
             ":py:class:`~collections.abc.Callable`\\[\\[:py:class:`int`, " ":py:class:`str`], :py:class:`bool`]",
         ),
         (Pattern, ":py:class:`~typing.Pattern`"),
@@ -470,6 +470,8 @@ def test_sphinx_output(
     # There should be a warning about an unresolved forward reference
     warnings = warning.getvalue().strip()
     assert "Cannot resolve forward reference in type annotations of " in warnings, warnings
+    # There should not be warnings about incorrect block endings.
+    assert "Field list ends without a blank line; unexpected unindent." not in warnings, warnings
 
     format_args = {}
     for indentation_level in range(2):
@@ -768,6 +770,129 @@ def test_sphinx_output(
            -[ Examples ]-
 
            Here are a couple of examples of how to use this function.
+
+        dummy_module.func_with_overload(a, b)
+
+           f does the thing. The arguments can either be ints or strings but they must both have the same type.
+
+           Parameters:
+              * **a** ("Union"["int", "str"]) -- The first thing
+
+              * **b** ("Union"["int", "str"]) -- The second thing
+
+           Return type:
+              "None"
+
+        class dummy_module.TestClassAttributeDocs
+
+           A class
+
+           code: "Optional"["CodeType"]
+
+              An attribute
+
+        dummy_module.func_with_examples_and_returns_after()
+
+           f does the thing.
+
+           -[ Examples ]-
+
+           Here is an example
+
+           Return type:
+              "int"
+
+           Returns:
+              The index of the widget
+
+        dummy_module.func_with_parameters_and_stuff_after(a, b)
+
+           A func
+
+           Parameters:
+              * **a** ("int") -- a tells us something
+
+              * **b** ("int") -- b tells us something
+
+           Return type:
+              "int"
+
+           More info about the function here.
+
+        dummy_module.func_with_rtype_in_weird_spot(a, b)
+
+           A func
+
+           Parameters:
+              * **a** ("int") -- a tells us something
+
+              * **b** ("int") -- b tells us something
+
+           -[ Examples ]-
+
+           Here is an example
+
+           Returns:
+              The index of the widget
+
+           More info about the function here.
+
+           Return type:
+              int
+
+        dummy_module.empty_line_between_parameters(a, b)
+
+           A func
+
+           Parameters:
+              * **a** ("int") --
+
+                One of the following possibilities:
+
+                * a
+
+                * b
+
+                * c
+
+              * **b** ("int") --
+
+                Whatever else we have to say.
+
+                There is more of it And here too
+
+           Return type:
+              "int"
+
+           More stuff here.
+
+        dummy_module.func_with_code_block()
+
+           A docstring.
+
+           You would say:
+
+              print("some python code here")
+
+           Return type:
+              "int"
+
+           -[ Examples ]-
+
+           Here are a couple of examples of how to use this function.
+
+        dummy_module.func_with_definition_list()
+
+           Some text and then a definition list.
+
+           Return type:
+              "int"
+
+           abc
+              x
+
+           xyz
+              something
         """
         expected_contents = dedent(expected_contents).format(**format_args).replace("–", "--")
         assert text_contents == maybe_fix_py310(expected_contents)
